/*******************************************************************
*  CLIDEMO.CXX
*  (c) 1992-1994 STAR DIVISION
*******************************************************************/

#include <fstream.h>

#ifdef MAC
#include <sysdep.hxx>
#endif

#include <sv.hxx>
#include <so.hxx>

#include "clidemo.hrc"

// --- class MyMainWindow ------------------------------------------

class MyMainWindow : public WorkWindow
{
public:
                    MyMainWindow() :
                        WorkWindow( NULL, WB_APP | WB_STDWORK ) {}

    virtual BOOL    Close();
};

// --- class MyClient ----------------------------------------------

class MyWindow : public MDIWindow
{
private:
    String          aFile;
    StarObject*     pObj;
    StarObject*     pUndo;
    BOOL            bCreate;

public:
                    MyWindow( StarObject* );
                    ~MyWindow();

    virtual BOOL    Close();
    virtual void    Resize();
    virtual void    Paint( const Rectangle& );
    virtual void    MouseButtonDown( const MouseEvent& );

    void            SetFilename( const String& rName )
                        { aFile = rName; }

    void            Undo();
    BOOL            CanUndo() const { return (BOOL)(pUndo != NULL); }
    void            DefActions( Menu* );
    void            Action( short nAction );
    void            Notify();
    BOOL            IsActive() const { return pObj->IsActive(); }

    void            Copy();
    void            Print();
    void            Save();
    void            SaveAs();
};

// --- class MyClient ----------------------------------------------

class MyClient : public MDIApplication
{
private:
    MyMainWindow*   pWindow;

public:
    void            Main( int Argc, char* Argv[] );

    long            SetupMenu( Menu* pMenu );
    long            MenuCommand( Menu* pMenu );
    BOOL            QueryFile( String&, BOOL );
    void            PasteObject();
    void            LoadObject();
};

// --- Instances of Application Class ------------------------------

MyClient aApp;
Printer* pPrn;

// --- MyClient::Main() --------------------------------------------

void MyClient::Main( int, char*[] )
{
#ifdef MAC
    Sysdepen::InitOLE();
    if ( !Sysdepen::IsOLEInit() )
        pApp->Abort("Unable to initialize OLE !");
#endif

    EnableSVLook();
    ChangeAppFont( Font( ResId( RID_APPFONT ) ) );

    MyMainWindow aMainWin;
    pWindow = &aMainWin;
    aMainWin.SetText( String( "StarObject-Client-Demo" ) );

    MenuBar aBar( ResId( IDM_MENU ) );
    PopupMenu* pFileMenu = aBar.GetPopupMenu( IDM_FILE );
    PopupMenu* pEditMenu = aBar.GetPopupMenu( IDM_EDIT );
    PopupMenu* pWinMenu  = aBar.GetPopupMenu( IDM_WINDOW );
    pFileMenu->PushActivateHdl( LINK( this, MyClient, SetupMenu ) );
    pEditMenu->PushActivateHdl( LINK( this, MyClient, SetupMenu ) );
    aBar.PushSelectHdl( LINK( this, MyClient, MenuCommand ) );
    ChangeAppMenu( &aBar );
    ChangeMDIMenu( pWinMenu );

    StarObject::ChangeAppName( "StarObject-Client" );

    Printer aPrn;
    pPrn = &aPrn;

    aMainWin.Show();

    Execute();
}

// --- MyClient::SetupMenu() ---------------------------------------

long MyClient::SetupMenu( Menu* pMenu )
{
    MyWindow* pCur = (MyWindow*)GetActiveWindow();

    BOOL bCur = BOOL( pCur != NULL );
    if ( bCur )
        bCur = !pCur->IsActive();

    pMenu->EnableItem( IDM_SAVE,  bCur );
    pMenu->EnableItem( IDM_SAVEAS,bCur );
    pMenu->EnableItem( IDM_PRINT, bCur );
    pMenu->EnableItem( IDM_COPY,  bCur );
    pMenu->EnableItem( IDM_PASTE, StarObject::HasClipboardData() );
    pMenu->EnableItem( IDM_UNDO, bCur && pCur->CanUndo() );
    if ( pCur )
    {
        if( pMenu->GetItemPos( IDM_ACTION ) == MENU_ITEM_NOTFOUND )
        {
            pMenu->InsertItem( IDM_ACTION, "" );
            pMenu->ChangePopupMenu( IDM_ACTION, new PopupMenu );
        }
        pCur->DefActions( pMenu );
    }
    else
        pMenu->RemoveItem( pMenu->GetItemPos( IDM_ACTION ) );
    return TRUE;
}

// --- MyClient::MenuCommand() -------------------------------------

long MyClient::MenuCommand( Menu* pMenu )
{
    MyWindow* pCur = (MyWindow*)GetActiveWindow();
    short n = pMenu->GetCurItemId();

    switch ( n )
    {
        case IDM_LOAD:
            LoadObject();
            break;
        case IDM_SAVE:
            pCur->Save();
            break;
        case IDM_SAVEAS:
            pCur->SaveAs();
            break;
        case IDM_PRINT:
            pCur->Print();
            break;
        case IDM_EXIT:
            pWindow->Close();
            break;

        case IDM_UNDO:
            pCur->Undo();
            break;
        case IDM_COPY:
            pCur->Copy();
            break;
        case IDM_PASTE:
            PasteObject();
            break;
        case IDM_INSOBJ:
            new MyWindow( NULL );
            break;

        case IDM_TILE:
            Tile();
            break;
        case IDM_CASCADE:
            Cascade();
            break;
        case IDM_HORIZONTAL:
            Horizontal();
            break;
        case IDM_VERTICAL:
            Vertical();
            break;
        case IDM_ARRANGE:
            Arrange();
            break;

        default:
            if( n > IDM_ACTION )
                pCur->Action( n - IDM_ACTION - 1 );
    }

    return TRUE;
}

// --- MyClient::PasteObject() ------------------------------------

void MyClient::PasteObject()
{
    if ( StarObject::HasClipboardData() )
    {
        StarObject* p = new StarObject;
        if ( p->Paste() )
        {
            p->ChangeName( "Object" );
            new MyWindow( p );
        }
        else
            ErrorBox( NULL, WB_OK,
                      "Object can not be created!" ).Execute();
    }
}

// --- MyClient::LoadObject() --------------------------------------

void MyClient::LoadObject()
{
    String aName( "*.OLE" );
    if ( QueryFile( aName, FALSE ) )
    {
#if defined( __ZTC__ ) || defined( UNX ) || defined( MAC )
        ifstream s( (const char*)aName, ios::in );
#else
        ifstream s( (const char*)aName, ios::in | ios::binary );
#endif
        if ( s.good() )
        {
            StarObject* pObj = new StarObject;
            pObj->ChangeName( "Object" );
            s >> *pObj;
            if ( s.good() )
            {
                MyWindow* w = new MyWindow( pObj );
                w->SetFilename( aName );
                return;
            }
            else
                delete pObj;
        }

        ErrorBox( NULL, WB_OK, "Object can not be loaded!" ).Execute();
    }
}

// --- MyClient::QueryFile() ---------------------------------------

BOOL MyClient::QueryFile( String& rName, BOOL bSave )
{
    FileDialog aDlg( GetAppWindow(),
                     WB_SVLOOK | (bSave ? WB_SAVEAS : WB_OPEN) );
    aDlg.SetPath( rName );
    aDlg.SetDefaultExt( rName );
    String aFilter( "Objects" );
    aDlg.AddFilter( aFilter, rName );
    aDlg.SetCurFilter( aFilter );
    if ( aDlg.Execute() )
    {
        rName = aDlg.GetPath();
        return TRUE;
    }
    else
        return FALSE;
}

// --- MyMainWindow::Close() ---------------------------------------

BOOL MyMainWindow::Close()
{
    if ( aApp.CloseAll() )
        return WorkWindow::Close();
    else
        return TRUE;
}

// --- MyWindow::MyWindow() ----------------------------------------

MyWindow::MyWindow( StarObject* o ) :
            MDIWindow( (MDIWindow*)aApp.GetAppWindow() )
{
    bCreate = TRUE;
    pObj = o;
    pUndo = NULL;

    if ( !o )
    {
        pObj = new StarObject;
        pObj->ChangeName( "Object" );
        pObj->Create( String( "New Object" ) );
    }
    else
    {
        bCreate = TRUE;
        Size s( pObj->GetSize() );
        ChangeOutputSizePixel( LogicToPixel( s, MAP_100TH_MM ) );
        SetText( pObj->GetServerName() );
        Show();
    }

    pObj->ChangeNotifyHdl( LINK( this, MyWindow, Notify ) );
}

// --- MyWindow::~MyWindow() ---------------------------------------

MyWindow::~MyWindow()
{
    delete pUndo;
    delete pObj;
}

// --- MyWindow::Notify() -----------------------------------------

void MyWindow::Notify()
{
    BOOL bChanged = pObj->IsChanged();
    BOOL bShow = bCreate;
    StarObjectError nErr = pObj->GetError();

    if( nErr != SOERR_OK )
    {
        String aMsg;
        switch( nErr )
        {
            case SOERR_CREATE:
                aMsg = "Object can not be created!"; break;
            case SOERR_CREATED:
                aMsg = "Object is already created!"; break;
            case SOERR_ACTIVE:
                aMsg = "Object is already active!"; break;
            case SOERR_NO_SERVER:
                aMsg = "Object has no servername!"; break;
            case SOERR_START_SERVER:
                aMsg = "Server can not be activated!"; break;
            case SOERR_ACTION:
                aMsg = "Unknown actioncode!"; break;
            case SOERR_LAYER:
                aMsg = "Error in communication!"; break;
            default:
                aMsg = String( (short)nErr ); break;
        }
        ErrorBox( this, WB_OK, aMsg ).Execute();
    }
    else
    {
        if ( bChanged )
        {
            Size s( pObj->GetSize() );
            ChangeOutputSizePixel( LogicToPixel( s, MAP_100TH_MM ) );
            SetText( pObj->GetServerName() );
            if ( bShow )
                Show();
            else
                Invalidate();
        }
    }
}

// --- MyWindow::Copy() --------------------------------------------

void MyWindow::Copy()
{
    pObj->Copy();
}

// --- MyWindow::DefActions() --------------------------------------

void MyWindow::DefActions( Menu* pMenu )
{
    String s( "Object " );
    s += pObj->GetServerName();
    pMenu->SetItemText( IDM_ACTION, s );
    PopupMenu* pSub = pMenu->GetPopupMenu( IDM_ACTION );
    if( pSub )
    {
        short n = pSub->GetItemCount();
        for( short i = 0; i < n; i++)
            pSub->RemoveItem( 0 );
        n = 0;
        for ( ;; )
        {
            String s = pObj->GetActionText( n );
            if( !s.Len() ) break;
            pSub->InsertItem( IDM_ACTION + ++n, s );
        }
        pMenu->EnableItem( IDM_ACTION, BOOL( n != 0 ) );
    }
}

// --- MyWindow::Action() ------------------------------------------

void MyWindow::Action( short nAction )
{
    delete pUndo;
    pUndo = new StarObject;
    pUndo->Copy( *pObj );
    pObj->Action( nAction );
}

// --- MyWindow::Print() -------------------------------------------

void MyWindow::Print()
{
    aApp.Wait( TRUE );

    Size aObjSize = pPrn->LogicToPixel( pObj->GetSize(),
                                        MAP_100TH_MM );
    Size aPaperSize( pPrn->GetPaperSize() );
    pPrn->StartJob( GetText() );
    pPrn->StartPage();
    Point aX0( (aPaperSize.Width()  - aObjSize.Width()  ) / 2,
               (aPaperSize.Height() - aObjSize.Height() ) / 2 );
    pObj->Draw( *pPrn, aX0 );
    pPrn->EndPage();
    pPrn->EndJob();

    while( pPrn->IsPrinting() )
        aApp.Reschedule();

    aApp.Wait( FALSE );
}

// --- MyWindow::Close() -------------------------------------------

BOOL MyWindow::Close()
{
    delete this;
    return TRUE;
}

// --- MyWindow::Resize() ------------------------------------------

void MyWindow::Resize()
{
    if ( !bCreate && pObj )
    {
        pObj->ChangeSize( PixelToLogic( GetOutputSizePixel(),
                                        MAP_100TH_MM ) );
        Invalidate();
    }

    bCreate = FALSE;
}

// --- MyWindow::Paint() -------------------------------------------

void MyWindow::Paint( const Rectangle& )
{
    pObj->Draw( *this, Point( 0, 0 ) );
}

// --- MyWindow::Save() --------------------------------------------

void MyWindow::Save()
{
    if ( !aFile.Len() )
        SaveAs();
    else
    {
#if defined( __ZTC__ ) || defined( UNX ) || defined( MAC )
        ofstream s( (const char*)aFile, ios::out );
#else
        ofstream s( (const char*)aFile, ios::out | ios::binary );
#endif
        if ( s.good() )
            s << *pObj;
        s.close();

        if ( !s )
            ErrorBox( this, WB_OK, "Object is not persistent!" ).Execute();
    }
}

// --- MyWindow::SaveAs() ------------------------------------------

void MyWindow::SaveAs()
{
#ifdef MAC
    String aName( "OLE Test" );
#else
    String aName( "*.OLE" );
#endif

    if ( aApp.QueryFile( aName, TRUE ) )
    {
        aFile = aName;
        Save();
    }
}

// --- MyWindow::Undo() --------------------------------------------

void MyWindow::Undo()
{
    if ( pUndo )
    {
        delete pObj;
        pObj = pUndo;
        pUndo = NULL;

        ChangeOutputSizePixel( LogicToPixel( pObj->GetSize(),
                                             MAP_100TH_MM ) );
        Invalidate();
    }
}

// --- MyWindow::MouseButtonDown() ---------------------------------

void MyWindow::MouseButtonDown( const MouseEvent& aEvt )
{
    if ( aEvt.GetClicks() == 2 )
        Action( 0 );
}
